/***************************************************************************
                          kap_driverselectorpage.cpp  -  description
                             -------------------
    begin                : Tue Feb 8 2000
    copyright            : (C) 2000 by Michael Goffioul
    email                : goffioul@imec.be
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include <qlabel.h>
#include <qlistview.h>
#include <qlayout.h>
#include <qfileinfo.h>
#include <klocale.h>

#include "kap_driverselectorpage.h"
#include "kprinterdescription.h"
#include "kbuffer.h"


Kap_DriverSelectorPage::Kap_DriverSelectorPage(QWidget *parent, const char *name )
:Kap_WizardPage(parent,name)
{
	setTitle(i18n("Driver selection"));
        values_ = 0;

	QLabel	*explLabel_ = new QLabel(this);
        explLabel_->setText(i18n("<p>Several drivers have been detected for the selected printer. Select the driver you want to use. Note that you will have the opportunity to test it and to change it if needed.</p>"));

	drivers_ = new QListView(this);
        drivers_->addColumn(i18n("Driver"));
        drivers_->addColumn(i18n("File"));
        drivers_->setFrameStyle(QFrame::WinPanel|QFrame::Sunken);
        drivers_->setSorting(0);

	QVBoxLayout	*mainLayout_ = new QVBoxLayout(this, 10, 10);
        mainLayout_->addWidget(explLabel_, 0);
        mainLayout_->addWidget(drivers_, 1);
}

Kap_DriverSelectorPage::~Kap_DriverSelectorPage()
{
}

bool Kap_DriverSelectorPage::initialize()
{
	KPrinterDescription	*descr = KPrinterDescription::get();
        bool	comingBack = (descr->getPrinterValues() != 0);
        // re-initialize (useful when coming back and PNP local printer -> possible
        // to come back further to "Driver selction method" page and everything is
        // still OK).
        descr->setPrinterValues(0);
        if (comingBack)
        { // we're coming back, do nothing
        	return true;
        }
        Kap_ManuModelDico	*dico = Kap_ManuModelDico::get();
        drivers_->clear();
        values_ = 0;
        if (dico)
        {
                if (descr->isPnp()) values_ = dico->getDescriptionForPnp(descr->getManufacturer(), descr->getModel());
                else values_ = dico->getDescription(descr->getManufacturer(), descr->getModel());
		// if we get there, then values must exist and have more than one entry
                for (values_->first();values_->current();values_->next())
                {
                        QFileInfo	fi(values_->current()->getPpdFile());
                        new QListViewItem(drivers_, values_->current()->getNickName(), fi.fileName());
                }
                drivers_->setSelected(drivers_->itemAt(QPoint(5,5)), true);
                return true;
        }
	else return false;
}

Kap_Main::e_PageType Kap_DriverSelectorPage::getNextPage(QString&, bool)
{
        return Kap_Main::DriverConfig;
}
Kap_Main::e_PageType Kap_DriverSelectorPage::getType() const
{
	return Kap_Main::DriverSelector;
}

bool Kap_DriverSelectorPage::canNextPage(QString& msg)
{
        if (!values_)
        {
        	msg = "Internal error: you shouldn't be there";
                return false;
        }
        if (!drivers_->currentItem())
        {
        	msg = i18n("You must first select a driver !");
                return false;
        }
        QString	nick_ = drivers_->currentItem()->text(0);
        for (values_->first();values_->current();values_->next())
        	if (values_->current()->getNickName() == nick_) break;
	if (!values_->current())
        {
        	msg = "Internal error: driver not found";
                return false;
        }

        Kap_ModelItem	*value = values_->current();
        KPrinterDescription	*parDet = KPrinterDescription::get();
//        Kap_ManuModelDico	*dico = Kap_ManuModelDico::get();
	parDet->setPrinterValues( value );
	parDet->setDescription( value->getPnpDescr() );
        parDet->setNickName( value->getNickName() );
/*	KBuffer	*buffer = new KBuffer;
	if (dico->getValidPpdFileAsBuffer(*value,*buffer) != 0)
	{
		parDet->setBuffer(buffer);
	}
	else
	{
		msg = i18n("Unable to find a driver file for this model !");
		return false;
	}*/
	return true;
}
