
#include "kap_smbselector.h"
#include "kap_smbutil.h"
#include "kprinterdescription.h"
#include "kups.h"

#include <qlabel.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qlistview.h>
#include <qheader.h>
#include <qmessagebox.h>
#include <kiconloader.h>
#include <kapp.h>

QListViewItem* findGroupItem(const QString& groupname, QListViewItem *first)
{
	QListViewItem	*item(first);
	while (item)
		if (item->text(0) == groupname) break;
		else item = item->nextSibling();
	return item;
}

//-----------------------------------------------------------------------------

class SmbServerItem : public QListViewItem
{
public:
	SmbServerItem(QListViewItem *parent, const char *name, SmbInfo *info)
		: QListViewItem(parent,name), Info(info) {}
	SmbInfo* getInfo() const { return Info; }
private:
	SmbInfo	*Info;
};

//-----------------------------------------------------------------------------

Kap_SmbSelector::Kap_SmbSelector(QWidget *parent, const char *name)
	: Kap_WizardPage(parent,name)
{
	setTitle(i18n("SMB printer selection"));

	// widget creation
	smblistlabel_ = new QLabel(i18n("Choose a printer :"),this);

	smblist_ = new QListView(this);
	smblist_->setFrameStyle(QFrame::WinPanel|QFrame::Sunken);
	smblist_->addColumn("SMB");
	smblist_->header()->hide();
	smblist_->setRootIsDecorated(true);
	connect(smblist_,SIGNAL(doubleClicked(QListViewItem*)),SLOT(printersRequested(QListViewItem*)));
	connect(smblist_,SIGNAL(selectionChanged(QListViewItem*)),SLOT(printerSelected(QListViewItem*)));

	scan_ = new QPushButton(i18n("Scan network..."),this);
	connect(scan_,SIGNAL(clicked()),SLOT(scanClicked()));

	workgrouplabel_ = new QLabel(i18n("Workgroup :"),this);
	serverlabel_ = new QLabel(i18n("Server :"),this);
	printerlabel_ = new QLabel(i18n("Printer :"),this);

	workgroup_ = new QLineEdit(this);
	server_ = new QLineEdit(this);
	printer_ = new QLineEdit(this);
}

Kap_SmbSelector::~Kap_SmbSelector()
{
}

void Kap_SmbSelector::resizeEvent(QResizeEvent *)
{
	int	w((width()-20)/2);
	smblistlabel_->setGeometry(5,5,w,25);
	int	h(height()-70);
	smblist_->setGeometry(5,30,w,h);
	scan_->setGeometry(5,35+h,w,30);
	int	x(w+15),y(30),s((h-150)/2);
	workgrouplabel_->setGeometry(x,y,w,25);
	workgroup_->setGeometry(x,y+25,w,25);
	y += (50+s);
	serverlabel_->setGeometry(x,y,w,25);
	server_->setGeometry(x,y+25,w,25);
	y += (50+s);
	printerlabel_->setGeometry(x,y,w,25);
	printer_->setGeometry(x,y+25,w,25);
}

Kap_Main::e_PageType Kap_SmbSelector::getType() const
{
	return Kap_Main::SmbSelector;
}

Kap_Main::e_PageType Kap_SmbSelector::getNextPage(QString&, bool fake)
{
	if (fake) return Kap_Main::ModelSelector;

	KPrinterDescription	*descr = KPrinterDescription::get();
	descr->setSMBInfo(QString(workgroup_->text()),QString(server_->text()),QString(printer_->text()));
	Kap_SmbUtil::release();
	return Kap_Main::ModelSelector;
}

bool Kap_SmbSelector::canNextPage(QString& msg)
{
	if (QString(server_->text()).isEmpty() || QString(printer_->text()).isEmpty())
	{
		msg = i18n("You must specify at least a server and a printer.");
		return false;
	}
	return true;
}

void Kap_SmbSelector::scanClicked()
{
	smblist_->setCursor(waitCursor);
	smblist_->clear();
	Kap_SmbUtil	*util = Kap_SmbUtil::get();
	QList<SmbInfo>	infos = util->getSmbList(true);
	QListIterator<SmbInfo>	it(infos);
	for (;it.current();++it)
	{
		QListViewItem	*groupitem = findGroupItem(it.current()->Workgroup.stripWhiteSpace(),smblist_->firstChild());
		if (!groupitem)
		{
			groupitem = new QListViewItem(smblist_,it.current()->Workgroup.stripWhiteSpace().data());
			groupitem->setSelectable(false);
			groupitem->setPixmap(0,SmallIcon("kups_group"));
		}
		QListViewItem	*serveritem = new SmbServerItem(groupitem,it.current()->Server.stripWhiteSpace().data(),it.current());
		serveritem->setPixmap(0,SmallIcon("kups_computer"));
		
	}
	smblist_->setCursor(arrowCursor);
}

void Kap_SmbSelector::printersRequested(QListViewItem *item)
{
	if (item->depth() == 1)
	{
#ifdef DEBUG
debug("request printers for %s",item->text(0).data() );
#endif
		KPrinterDescription	*descr = KPrinterDescription::get();
		SmbInfo		*info = ((SmbServerItem*)(item))->getInfo();
		if (info->PrinterList.count() == 0)
		{
			smblist_->setCursor(waitCursor);
			Kap_SmbUtil	*util = Kap_SmbUtil::get();
			if (util->getPrinters(info,descr->getUser(),descr->getPassword()))
			{
				QStrListIterator	it(info->PrinterList);
				for (;it.current();++it)
				{
					QListViewItem	*printeritem = new QListViewItem(item,it.current(),info->Server.stripWhiteSpace().data(),info->Workgroup.stripWhiteSpace().data());
					printeritem->setPixmap(0,SmallIcon("kups_printer"));
				}
				if (info->PrinterList.count() == 0)
					QMessageBox::information(this,i18n("Printer list"),i18n("No printer found."),QMessageBox::Ok | QMessageBox::Default,0);
			}
			smblist_->setCursor(arrowCursor);
			item->setOpen(true);
		}
	}
}

void Kap_SmbSelector::printerSelected(QListViewItem *item)
{
	if (item->depth() == 2)
	{
		workgroup_->setText(item->text(2));
		server_->setText(item->text(1));
		printer_->setText(item->text(0));
	}
}
