
#include "kap_socketselector.h"
#include "kap_socketutil.h"
#include "kprinterdescription.h"
#include "kups.h"

#include <qlistview.h>
#include <qheader.h>
#include <qpushbutton.h>
#include <qprogressbar.h>
#include <qlineedit.h>
#include <qlabel.h>
#include <qmessagebox.h>
#include <kapp.h>
#include <kiconloader.h>



Kap_SocketSelector::Kap_SocketSelector(QWidget *parent, const char *name)
	: Kap_WizardPage(parent,name)
{
	setTitle(i18n("Network printer informations"));

	util_ = new Kap_SocketUtil();

	printerlist_ = new QListView(this);
	printerlist_->addColumn("");
	printerlist_->header()->hide();
	printerlist_->setFrameStyle(QFrame::WinPanel|QFrame::Sunken);

	printerlabel_ = new QLabel(i18n("Printer address"),this);
	portlabel_ = new QLabel(i18n("Port"),this);

	printer_ = new QLineEdit(this);
	port_ = new QLineEdit(this);

	scanbar_ = new QProgressBar(this);

	settings_ = new QPushButton(i18n("Settings"),this);
	scan_ = new QPushButton(i18n("Scan"),this);

	connect(printerlist_,SIGNAL(selectionChanged(QListViewItem*)),SLOT(printerClicked(QListViewItem*)));
	connect(scan_,SIGNAL(clicked()),SLOT(scanClicked()));
	connect(settings_,SIGNAL(clicked()),SLOT(settingsClicked()));
}

Kap_SocketSelector::~Kap_SocketSelector()
{
	delete util_;
}

Kap_Main::e_PageType Kap_SocketSelector::getType() const
{
	return Kap_Main::SocketSelector;
}

Kap_Main::e_PageType Kap_SocketSelector::getNextPage(QString&, bool fake)
{
	if (fake) return Kap_Main::ModelSelector;

	int	p(9100);
	QString	port(port_->text());
	if (!port.isEmpty()) p = port.toInt();
	KPrinterDescription	*descr = KPrinterDescription::get();
	descr->setSocketInfo(QString(printer_->text()),p);
	return Kap_Main::ModelSelector;
}

bool Kap_SocketSelector::canNextPage(QString& msg)
{
	if (QString(printer_->text()).isEmpty())
	{
		msg = i18n("You must enter a printer address !");
		return false;
	}
	QString	port(port_->text());
	int	p(9100);
	if (!port.isEmpty())
	{
		bool	ok;
		p = port.toInt(&ok);
		if (!ok)
		{
			msg = i18n("Wrong port number !");
			return false;
		}
	}
	if (!util_->checkPrinter(printer_->text(),p))
	{
		msg = i18n("No printer found at this address/port !");
		return false;
	}
	return true;
}

void Kap_SocketSelector::resizeEvent(QResizeEvent*)
{
	int	w((width()-20)/2);
	QSize	sb(settings_->sizeHint()), sl(printerlabel_->sizeHint()), se(printer_->sizeHint());
	int	h(height()-40-sb.height());
	printerlist_->setGeometry(5,15,w,h);
	int	s((h-2*se.height()-2*sl.height())/3), y(10+s);
	printerlabel_->setGeometry(15+w,y,w,sl.height());
	y += sl.height();
	printer_->setGeometry(15+w,y,w,se.height());
	y += (se.height()+s);
	portlabel_->setGeometry(15+w,y,w,sl.height());
	y += sl.height();
	port_->setGeometry(15+w,y,w,se.height());
	y = (20+h);
	int	w2((w-5)/2);
	settings_->setGeometry(5,y,w2,sb.height());
	scan_->setGeometry(w2+10,y,w2,sb.height());
	QSize	ss(scanbar_->sizeHint());
	int	sp(QMAX(0,(sb.height()-ss.height())/2));
	scanbar_->setGeometry(15+w,y+sp,w,ss.height());
}

void Kap_SocketSelector::settingsClicked()
{
	util_->configureScan(this);
}

void Kap_SocketSelector::scanClicked()
{
	if (!util_->scanNetwork(scanbar_))
		QMessageBox::critical(this,i18n("Error"),i18n("Network scan failed"),QMessageBox::Ok|QMessageBox::Default,0);
	else
	{
		printerlist_->clear();
		const QList<SocketInfo>	*list = util_->printerList();
		QListIterator<SocketInfo>	it(*list);
		for (;it.current();++it)
		{
			QString	num, name;
			num.setNum(it.current()->Port);
			if (it.current()->Name.isEmpty()) name.sprintf("<Unknown> (%s)",it.current()->IP.data());
			else name = it.current()->Name.data();
			QListViewItem	*item = new QListViewItem(printerlist_,name.data(),it.current()->IP.data(),num.data());
			item->setPixmap(0,SmallIcon("kups_printer"));
		}
	}
	scanbar_->reset();
}

void Kap_SocketSelector::printerClicked(QListViewItem *item)
{
	if (!item) return;
	printer_->setText(item->text(1));
	port_->setText(item->text(2));
}
