/***************************************************************************
                          kjob.cpp  -  description
                             -------------------
    begin                : Thu Oct 28 1999
    copyright            : (C) 1999 by jec
    email                : cuendet@linkvest.ch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kjob.h"

#include "kprinter.h"
#include "kjobhelper.h"
#include "kprintermanager.h"
#include "kups_ipprequest.h"
#include "kups_infos.h"


KJob::KJob()
:id_(-1),
name_(""),
owner_(0),
size_(-1),
userName_(""),
state_(IPP_JOB_ABORTED),
uri_("")
{
	helper_	= KJobHelper::get();
}
KJob::KJob(const KJob& other)
:KObject(other)
{
	copy(&other);
	helper_ = other.helper_;
}
KJob::~KJob()
{
}
	
void KJob::copy(const KJob *other)
{
	id_ = other->id_;
	size_ = other->size_;
	userName_ = other->userName_;
	state_ = other->state_;
	uri_ = other->uri_;

	if (owner_ != other->owner_)
	{
		// remove the job from old printer job list (without deleting it)
		if (owner_) owner_->removeJobFromList(this,false);
		owner_ = other->owner_;
		// add it to the new printer job list
		if (owner_) owner_->addJobToList(this);
	}
}

void KJob::setAttribute(ipp_attribute_t* attr)
{
	helper_->setAttribute(this, attr);
}

bool KJob::sendCommand(Command com)
{
	switch(com)
	{
	case Remove:
		return sendRemove();
	case RemoveAll:
		return sendRemoveAll();
	case Hold:
	case Release:
	case Restart:
		return helper_->sendJobCommand(this,com);
	default:
		break;
	};
	KPrinterManager::setLastError(QString("Unknown job command"));
	return false;
}

bool KJob::sendCommand(Command command, KPrinter *printer)
{
	return helper_->sendJobCommand(this,command,printer);
}

bool KJob::sendRemove()
{
	bool ret = true;
	Kups_IppRequest request;
	request.setOperation(IPP_CANCEL_JOB);

	request.setUri("job-uri", getUri());
	request.setName("requesting-user-name", Kups_Infos::get()->getUser().data() );
	if (request.doRequest("/jobs/"))
	{
		ret = true;
    if (request.getResponseStatus() == IPP_NOT_FOUND)
    {
    	QString error;
	    error.sprintf("Destination %s does not exist!\n", getPrinter()->getName().data());
      KPrinterManager::setLastError(error);
	    ret = false;
    }
	}
	else
	{
		ret = false;
	}

	return ret;
}

bool KJob::sendRemoveAll()
{
	KPrinterManager::setLastError(QString("This feature is not implemented yet. Sorry."));
	return false;
}
