#include "kpp_memberspage.h"

#include <qlabel.h>
#include <qlayout.h>
#include <qpushbutton.h>
#include <klocale.h>
#include <qstrlist.h>

#include "kprinterdescription.h"
#include "kaddprinterwizard.h"

Kpp_MembersPage::Kpp_MembersPage(QWidget *parent, const char *name)
	: QWidget(parent,name)
{
	desc_ = 0;
	modified_ = false;
	initWidget();
}

Kpp_MembersPage::~Kpp_MembersPage()
{
}

void Kpp_MembersPage::initWidget()
{
	infoLabel_ = new QLabel(this);
	infoLabel_->setAlignment(Qt::AlignTop|Qt::AlignLeft|Qt::ExpandTabs);

	QPushButton	*change_ = new QPushButton(i18n("Change..."), this);
	change_->setFixedSize(change_->sizeHint());
	connect(change_,SIGNAL(clicked()),SLOT(changeClicked()));

	QVBoxLayout	*mainLayout_ = new QVBoxLayout(this, 10);
	mainLayout_->addWidget(infoLabel_, 1);
	mainLayout_->addWidget(change_, 0, Qt::AlignLeft|Qt::AlignVCenter);
}

void Kpp_MembersPage::changeClicked()
{
	// fill static printer description first, so that the class selector is up-to-date
	QStrList	*prs = desc_->getPrintersList();
	if (prs)
	{
		QStrList	*newList = new QStrList(*prs);
		KPrinterDescription::get()->setPrintersList(newList);
	}
	// create the wizard
	Kap_AddPrinterWizard	wizard(this);
	wizard.setWizardControl(Kap_Main::ClassSelector, Kap_Main::ClassSelector, true);
	if (wizard.exec())
	{
		QStrList	*prs = KPrinterDescription::get()->getPrintersList();
		// update printer description
		if (prs)
		{
			QStrList	*newList = new QStrList(*prs);
			desc_->setPrintersList(newList);
		}

		modified_ = true;

		// update view
		setInfos(desc_);
	}
}

void Kpp_MembersPage::collectInfos(KPrinterDescription *d)
{
	QStrList	*prs = desc_->getPrintersList();
	if (prs)
	{
		QStrList	*newList = new QStrList(*prs);
		d->setPrintersList(newList);
	}
	d->setType(KPrinterManager::e_Class);
}

void Kpp_MembersPage::setInfos(KPrinterDescription *d)
{
	desc_ = d;
	QString	str = i18n("Members: ");
	QStrList	*prs = d->getPrintersList();
	if (prs)
	{
		for (prs->first();prs->current();prs->next())
		{
			str.append(prs->current());
			str.append(", ");
		}
		str.truncate(str.length()-2);
	}
	infoLabel_->setText(str);
	infoLabel_->setMinimumHeight(infoLabel_->sizeHint().height());
}
