/***************************************************************************
                          kprinter.h  -  description
                             -------------------
    begin                : Thu Oct 28 1999
    copyright            : (C) 1999 by jec
    email                : cuendet@linkvest.ch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KPRINTER_H
#define KPRINTER_H


#include <cups/ipp.h>

#include <qobject.h>
#include <qlist.h>
#include <qstrlist.h>

#include "kobject.h"
#include <qstring.h>


class KJob;
class KBuffer;
class KPrinterDescription;
class KPrinterHelper;
class KClassHelper;

class KPrinter : public KObject
{
	Q_OBJECT
public:

	enum State { Idle, Processing, Stopped };
	enum Orientation { Landscape, Portrait, Rev_Landscape, Rev_Portrait };
	
	enum Command { AsDefault, Enable, Disable, Remove, RemoveFromClass, AddToClass };
	
  KPrinter();
  KPrinter(const KPrinter& other);
  virtual ~KPrinter() = 0;

  void copy(const KPrinter *printer);
//  virtual KPrinter& copy(const KPrinter& other);

  // Commands
  virtual bool sendCommand(Command command) = 0;
  virtual bool sendCommand(Command command, KPrinter* other) = 0;

  virtual bool printBuffer(const KBuffer& page) = 0;
  virtual bool printFile(const QString& page) = 0;

  virtual void fillDescription(KPrinterDescription *desc);

  static int getNbColumns();
  static QString getStaticColumnNameAt(int i);
  QString getColumnNameAt(int i);
  QString getColumnValueAt(int i);
  QString getColumnNameOrValueAt(int i, bool name);

  int getNbJobs() const;
  KJob* getJobAt(int i);
  void addJobToList(KJob* job);
  void removeJobFromList(KJob* job, bool remove = true);
  KJob* getJobByID(int id);

  void setName(const QString& name) { name_ = name; };
  const QString& getName() const { return name_; };

  void setDescription(const QString& descr) { description_ = descr; };
  const QString& getDescription() const { return description_; };

  bool isDefault() const { return isDefault_; };
  void setDefault(bool val) { isDefault_ = val; };

  void setState(State state) { state_ = state; };
  State getState() const { return state_; };

  void setOwner(const QString& str) { owner_ = str; };
  const QString& getOwner() const { return owner_; };

  void setLocation(const QString& str) { location_ = str; };
  const QString& getLocation() const { return location_; };

  void setInfo(const QString& str) { info_ = str; };
  const QString& getInfo() const { return info_; };

  void setModelInfo(const QString& str) { modelinfo_ = str; };
  const QString& getModelInfo() const { return modelinfo_; };

  void setLocalURI(const QString& str) { localURI_ = str; };
  const QString& getLocalURI() const { return localURI_; };

  void addIppURI(const QString& str) { ippURIs_.append(str); }
  void removeIppURI(const QString& str) { ippURIs_.remove(str); }
  const QString& getIppURI(int index = 0) const { return *(ippURIs_.at(index));}
  bool supportIppURI(const QString& uri) const { return (ippURIs_.find(uri) != ippURIs_.end()); }
  //void setIppURI(const QString& str) { ippURI_ = str; };
  //const QString& getIppURI() const { return ippURI_; };

	void setStateReason(const QString& r) { stateReason_ = r; };
	const QString& getStateReason() const { return stateReason_; };

  void setIsColor(bool color) { isColor_ = color; };
  bool isColor() const { return isColor_; };

  void addMedia(const QString& str);
  void setDefaultMedia(const QString& str) { defaultMedia_ = str; };

  void addBanner(const QString& str) { if (supportedBanners_.find(str) == supportedBanners_.end()) supportedBanners_.append(str); }
  bool supportBanner(const QString& str) { return (supportedBanners_.find(str) != supportedBanners_.end()); }
  const QStringList& getSupportedBanners() { return supportedBanners_; }
  void setStartBanner(const QString& str) { startBanner_ = str; }
  const QString& getStartBanner() const { return startBanner_; }
  void setEndBanner(const QString& str) { endBanner_ = str; }
  const QString& getEndBanner() const { return endBanner_; }

  void setPrinterType(int type) { printerType_ = type; };
  int getPrinterType() const { return printerType_; };
  virtual bool isRemote() const = 0;
  virtual bool isImplicit() const { return false; }
  virtual bool isClass() const;

  void setIsAcceptingJobs(bool accepting) { isAcceptingJobs_ = accepting; };
  bool isAcceptingJobs() const { return isAcceptingJobs_; } ;

	void setDefaultCopies(int copies) { defaultCopies_ = copies; };
	int getDefaultCopies() const { return defaultCopies_; };

	void setCopiesRange(int min, int max) { copiesRangeMin_ = min; copiesRangeMax_ = max; };
	void setPageRangeSupported(bool sup) { pageRangeSupported_ = sup; };

  void setDefaultOrientation(Orientation orient) { defaultOrientation_ = orient; };
  Orientation getDefaultOrientation() const { return defaultOrientation_; };
	void addOrientation(Orientation orient);

private:
//  KPrinter& scopy(const KPrinter& other);

private:
  QString name_;
  QString description_;
  QString owner_;
  State state_;
  bool isDefault_;
  QString location_;
  QString info_;
  QString modelinfo_;
  QString localURI_;
  QStringList ippURIs_;
  bool isColor_;
  QStringList supportedMedia_;
  QString defaultMedia_;
  int printerType_;
  bool isAcceptingJobs_;
  int defaultCopies_;
  int copiesRangeMin_;
  int copiesRangeMax_;
  bool pageRangeSupported_;
  Orientation defaultOrientation_;
  QString stateReason_;

  QStringList mediaList_;
  QValueList<int> orientationList_;

  QList<KJob> jobs_;

  QString nbJobStr_;
  QString stateStr_;
  QString orientStr_;

  QStringList supportedBanners_;
  QString startBanner_;
  QString endBanner_;
};



class KRealPrinter : public KPrinter
{
	Q_OBJECT
public:
	KRealPrinter();
	KRealPrinter(const KRealPrinter& other);
	~KRealPrinter();

  virtual bool sendCommand(Command command);
  virtual bool sendCommand(Command command, KPrinter *other);

  virtual bool printBuffer(const KBuffer& page);
  virtual bool printFile(const QString& page);

	void setAttribute(ipp_attribute_t* attr);

	virtual bool isRemote() const;
	virtual bool isImplicit() const;

	virtual void fillDescription(KPrinterDescription *desc);

private:
	KPrinterHelper* helper_;
};


class KClassPrinter : public KPrinter
{
	Q_OBJECT
public:
	KClassPrinter();
	KClassPrinter(const KClassPrinter& other);
	~KClassPrinter();

	void copy(const KClassPrinter *classes);
	
	void addPrinterToClass(const QString& str);
	void addPrinterToClass(KPrinter* printer);

	int getNbPrinters() const;
	const KPrinter* getPrinterAt(int i) const;
	KPrinter* getPrinterAt(int i);
	bool hasPrinter(KPrinter *p) { return (printersList_.find(p) != -1); }
		
  virtual bool sendCommand(Command command);
  virtual bool sendCommand(Command command, KPrinter* other);

  virtual bool printBuffer(const KBuffer& page);
  virtual bool printFile(const QString& page);

	void setAttribute(ipp_attribute_t* attr);

	virtual bool isRemote() const;
	virtual bool isClass() const;

	virtual void fillDescription(KPrinterDescription *desc);

private:
	KClassHelper* helper_;
	QList<KPrinter> printersList_;
};

#endif
