#include "kprinterinstance.h"

#include <klocale.h>
#include <qstringlist.h>
#include <qtextstream.h>
#include <stdlib.h>

KPrinterInstance::KPrinterInstance()
{
	printer_ = "";
	isDefault_ = false;
	instance_ = "";
	options_.setAutoDelete(true);
}

KPrinterInstance::KPrinterInstance(const QString& printer, const QString& instance)
: printer_(printer), instance_(instance)
{
	isDefault_ = false;
	options_.setAutoDelete(true);
}

KPrinterInstance::~KPrinterInstance()
{
}

void KPrinterInstance::copy(const KPrinterInstance& p)
{
	printer_ = p.printer_;
	instance_ = p.instance_;
	isDefault_ = p.isDefault_;
	QDictIterator<KInstanceOption>	it(p.options_);
	for (;it.current();++it)
	{
		KInstanceOption	*opt = options_.find(it.currentKey());
		if (opt) opt->value_ = it.current()->value_;
		else options_.insert(it.currentKey(), new KInstanceOption(*(it.current())));
	}
}

const QString& KPrinterInstance::getPrinter()
{
	return printer_;
}

const QString& KPrinterInstance::getInstance() const
{
	return instance_;
}

void KPrinterInstance::setCupsDest(cups_dest_t *d)
{
	printer_ = d->name;
	instance_ = d->instance;
	if (instance_.isEmpty()) instance_ = i18n("(Default)");
	isDefault_ = (d->is_default != 0);
	options_.clear();
	for (int i=0;i<d->num_options;i++)
		options_.insert(QString(d->options[i].name), new KInstanceOption(d->options[i].name,d->options[i].value));
}

cups_dest_t* KPrinterInstance::getCupsDest()
{
	cups_dest_t	*d = (cups_dest_t*)malloc(sizeof(cups_dest_t));

	d->name = strdup(printer_.data());
	if (!instance_.isEmpty()) d->instance = strdup(instance_.data());
	else d->instance = NULL;
	d->is_default = (isDefault_ ? 1 : 0);
	d->num_options = 0;
	d->options = NULL;
	QDictIterator<KInstanceOption>	it(options_);
	for (;it.current();++it)
	{
		d->num_options = cupsAddOption(it.current()->option_.data(), it.current()->value_.data(), d->num_options, &(d->options));
	}

	return d;
}

bool KPrinterInstance::loadFromString(const QString& line)
{
	QStringList	words = QStringList::split(' ', line, false);
	int	p;
	if (words.count() < 2) return false;
	// Default ?
	isDefault_ = (words[0].lower() == "default");
	// Printer/Instance
	if ((p=words[1].find('/')) != -1)
	{
		printer_ = words[1].left(p);
		instance_ = words[1].right(words[1].length()-p-1);
	}
	else
	{
		printer_ = words[1];
		instance_ = i18n("(Default)");
	}
	// Options
	options_.clear();
	QString	s1, s2;
	for (uint i=2;i<words.count();i++)
	{
		if ((p=words[i].find('=')) != -1)
		{
			s1 = words[i].left(p).stripWhiteSpace();
			s2 = words[i].right(words[i].length()-p-1).stripWhiteSpace();
		}
		else
		{
			s1 = words[i];
			s2 = "";
		}
		options_.insert(s1, new KInstanceOption(s1,s2));
	}
	return true;
}

void KPrinterInstance::writeToString(QString& line)
{
	line = "";
	QTextStream	t(&line, IO_WriteOnly);
	t << (isDefault_ ? "Default " : "Dest ");
	t << printer_;
	if (!instance_.isEmpty() && instance_ != i18n("(Default)")) t << '/' << instance_;
	t << ' ';
	QDictIterator<KInstanceOption>	it(options_);
	for (;it.current();++it)
	{
		t << it.current()->option_;
		if (!it.current()->value_.isEmpty()) t << '=' << it.current()->value_;
		t << ' ';
	}
	line.truncate(line.length()-1);
}
