/***************************************************************************
                          kprintermanager.cpp  -  description
                             -------------------
    begin                : Thu Oct 28 1999
    copyright            : (C) 1999 by jec
    email                : cuendet@linkvest.ch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <klocale.h>

#include "kprintermanager.h"
#include "kprinterobserver.h"
#include "kjob.h"
#include "kprinter.h"
#include "kprinterdescription.h"
#include "kups_ipprequest.h"
#include "kups_infos.h"
#include "kbuffer.h"
#include "kups_attributesiterator.h"
#include "kinstancemanager.h"


// static vars
KPrinterManager* KPrinterManager::unique_ = 0;
QString KPrinterManager::lastError_;
QString KPrinterManager::user_;
QString KPrinterManager::password_;


// class KPrinterManager
KPrinterManager::KPrinterManager()
: notifyOn_(true), http_(0)
{
}

KPrinterManager::~KPrinterManager()
{
}

KPrinterManager* KPrinterManager::get()
{
	if (unique_ == 0)
	{
		unique_ = new KPrinterManager;
	}
	return unique_;
}
void KPrinterManager::setLastError(const QString& le)
{
	lastError_ = le;
}
const QString& KPrinterManager::getLastError()
{
	return lastError_;
}

void KPrinterManager::setAllPrintersDiscarded(bool disc)
{
	for (int i=0;i<getNbPrinters();i++)
	{
		KRealPrinter* p = getPrinterAt(i);
		p->setIsDiscarded(disc);
	}
}
void KPrinterManager::deleteAllDiscardedPrinters()
{
	for (int i=0;i<getNbPrinters();i++)
	{
		KRealPrinter* p = getPrinterAt(i);
		if (p->isDiscarded())
		{
			removePrinterToList(p);
			delete p;
			i--;
		}
	}
}

void KPrinterManager::setAllClassesDiscarded(bool disc)
{
	for (int i=0;i<getNbClasses();i++)
	{
		KPrinter* p = getClassAt(i);
		p->setIsDiscarded(disc);
	}
}
void KPrinterManager::deleteAllDiscardedClasses()
{
	for (int i=0;i<getNbClasses();i++)
	{
		KClassPrinter* c = getClassAt(i);
		if (c->isDiscarded())
		{
			removeClassToList(c);
			delete c;
			i--;
		}
	}
}

void KPrinterManager::setAllJobsDiscarded(bool disc)
{
	for (int i=0;i<getNbPrinters();i++)
	{
		KPrinter* p = getPrinterAt(i);
		for (int k=0;k<p->getNbJobs();k++)
		{
			KJob* j = p->getJobAt(k);
			j->setIsDiscarded(disc);			
		}
	}
	
	for (int i=0;i<getNbClasses();i++)
	{
		KPrinter* c = getClassAt(i);
		for (int k=0;k<c->getNbJobs();k++)
		{
			KJob* j = c->getJobAt(k);
			j->setIsDiscarded(disc);			
		}
	}
}
void KPrinterManager::deleteAllDiscardedJobs()
{
	for (int i=0;i<getNbPrinters();i++)
	{
		KPrinter* p = getPrinterAt(i);
		for (int k=0;k<p->getNbJobs();k++)
		{
			KJob* j = p->getJobAt(k);
			if (j->isDiscarded())
			{
				p->removeJobFromList(j);
			}
		}
	}
	
	for (int i=0;i<getNbClasses();i++)
	{
		KPrinter* c = getClassAt(i);
		for (int k=0;k<c->getNbJobs();k++)
		{
			KJob* j = c->getJobAt(k);
			if (j->isDiscarded())
			{
				c->removeJobFromList(j);
			}
		}
	}
}

bool KPrinterManager::initialize()
{
	return refresh();
}

void KPrinterManager::addObserver(KPrinterObserver* obs)
{
	observersList_.append(obs);
}

void KPrinterManager::notifyObservers()
{
	if (!notifyOn_) return;	// blocks the window refresh (useful when creating temp printer).
        for (int i=0;i<(int)observersList_.count();i++)
	{
		KPrinterObserver* obs = observersList_.at(i);
		obs->updateNow();
	}
}

int KPrinterManager::getNbClasses()
{
	return classes_.count();
}
KClassPrinter* KPrinterManager::getClassAt(int i)
{
	return  classes_.at(i);
}
KClassPrinter* KPrinterManager::getClassByName(const QString& name)
{
	for (int i=0;i<(int)classes_.count();i++)
	{
		KClassPrinter* classe = classes_.at(i);
		if (classe->getName() == name)
		{
			return classe;
		}
	}
	return 0;
}

KJob* KPrinterManager::getJobByID(int id)
{
	for (int i=0;i<getNbPrinters();i++)
	{
		KPrinter* printer = getPrinterAt(i);
		KJob* job = printer->getJobByID(id);
		if (job != 0)
		{
			return job;
		}
	}
	
	for (int i=0;i<getNbClasses();i++)
	{
		KPrinter* printer = getClassAt(i);
		KJob* job = printer->getJobByID(id);
		if (job != 0)
		{
			return job;
		}
	}
	return 0;
}

int KPrinterManager::getNbPrinters()
{
	return printers_.count();
}
KRealPrinter* KPrinterManager::getPrinterAt(int i)
{
	return printers_.at(i);
}
KRealPrinter* KPrinterManager::getPrinterByName(const QString& name)
{
	for (int i=0;i<(int)printers_.count();i++)
	{
		KRealPrinter* printer = printers_.at(i);
		if (printer->getName() == name)
		{
			return printer;
		}
	}
	return 0;
}

KRealPrinter* KPrinterManager::getPrinterByUri(const QString& uri, bool tryWithName)
{
	QString	suri(uri);
	if (suri.length() > 5 && suri.left(4) == "http") suri.replace(0,4,QString("ipp"));
	for (int i=0;i<(int)printers_.count();i++)
	{
		KRealPrinter	*pr = printers_.at(i);
		if (pr->supportIppURI(suri))
		{
			return pr;
		}
	}

	if (tryWithName)
	{
		int	index = uri.findRev('/');
		if (index != -1) return getPrinterByName(uri.right(uri.length()-index-1));
	}
	return 0;
/*
	int index;
	int index2;

	index = 0;
	index2 = uri.find("://", index);
	QString protocol = uri.mid(index, index2-index);
	
	index = index2+3;
	index2 = uri.find(':', index);
	QString host = uri.mid(index, index2-index);

	index = index2+1;
	index2 = uri.find('/', index);	
	QString port = uri.mid(index, index2-index);

	index = index2+1;
	index = uri.find('/', index);
	index++;
	index2 = uri.length();
	QString name = uri.mid(index, index2);
	
	return getPrinterByName(name);*/
}

KClassPrinter* KPrinterManager::getClassByUri(const QString& uri, bool tryWithName)
{
	QString	suri(uri);
	if (suri.length() > 5 && suri.left(4) == "http") suri.replace(0,4,QString("ipp"));
	for (int i=0;i<(int)classes_.count();i++)
	{
		KClassPrinter	*cl = classes_.at(i);
		if (cl->supportIppURI(suri))
		{
			return cl;
		}
	}

	if (tryWithName)
	{
		int	index = uri.findRev('/');
		if (index != -1) return getClassByName(uri.right(uri.length()-index-1));
	}
	return 0;
}

bool KPrinterManager::removePrinterToList(KRealPrinter* printer)
{
	printers_.remove(printer);
	return true;
}
bool KPrinterManager::addPrinterToList(KRealPrinter* printer)
{
	printers_.append(printer);
	return true;
}
bool KPrinterManager::removeClassToList(KClassPrinter* printer)
{
	classes_.remove(printer);
	return true;
}
bool KPrinterManager::addClassToList(KClassPrinter* printer)
{
	classes_.append(printer);
	return true;
}

void KPrinterManager::setDefaultPrinterUri(const QString& uri)
{
	KPrinter* p = getPrinterByUri(uri,true);
	if (p == 0)
	{
		p = getClassByUri(uri,true);
	}
	
	setDefaultPrinter(p);
}

void KPrinterManager::setDefaultPrinterName(const QString& name)
{
	KPrinter* p = getPrinterByName(name);
	if (p == 0)
	{
		p = getClassByName(name);
	}
	
	setDefaultPrinter(p);
}

void KPrinterManager::setDefaultPrinter(KPrinter* printer)
{
	if (printer != 0)
	{
		printer->setDefault(true);
	}
}

//--------------------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------------------

bool KPrinterManager::addPrinter(KPrinterDescription& d, bool update)
{
	if (!update) notifyOn_ = false;
        bool	val(false);
	switch( d.getType() )
	{
		case e_Class:
			val = addClass(d);
                        break;
		case e_Parallel:
		case e_Serial:
		case e_LPD:
		case e_Socket:
		case e_SMB:
		case e_USB:
	  case e_IPP:
		case e_File:
			val = addReal(d);
                        break;

		default:
			setLastError("Wrong type of printer to install!");
			break;
	}
	notifyOn_ = true;
	return val;
}

bool KPrinterManager::modifyPrinter(KPrinter *printer, KPrinterDescription *descr)
{
        if (!printer || !descr)
	{
		setLastError("KPrinterManager::modifyPrinter : wrong arguments");
		return false;
	}

	if (printer->getName() != descr->getName())
	{ // name changed, we need to removed old printer before adding the new one
		if (!printer->sendCommand(KPrinter::Remove))
			return false;
	}
	if (descr->getType() == e_Class) return addClass(*descr);
	else return addReal(*descr);
}

//---------------------------------------------------------------------------
bool KPrinterManager::addClass(KPrinterDescription& descr)
{
	QStrList* list = descr.getPrintersList();
	QString	uri;
	QStringList	uris;
	Kups_IppRequest	request;

	// class uri
	request.setOperation(CUPS_ADD_CLASS);
	uri = Kups_Infos::get()->getServerPortIppUri() + "/classes/" + descr.getName();
	request.setUri("printer-uri",uri);

	// members uri
	if (list)
	{
		for (int i=0;i<(int)(list->count());i++)
			uris += (Kups_Infos::get()->getServerPortIppUri() + "/printers/" + QString(list->at(i)));
		request.setUris("member-uris",uris);
	}

	// other infos
	request.setInteger(IPP_TAG_PRINTER, "printer-state", IPP_PRINTER_IDLE);
	request.setBoolean(IPP_TAG_PRINTER, "printer-is-accepting-jobs", 1);
 	if (!descr.getDescription().isNull()) request.setText(IPP_TAG_PRINTER, "printer-info", descr.getDescription() );
 	if (!descr.getLocation().isNull()) request.setText(IPP_TAG_PRINTER, "printer-location", descr.getLocation() );

	// send the request
	if (!request.doRequest("/admin/")) return false;

	// update KUPS view
	refresh();
	// enable the new class
	KPrinter* p = getClassByName(descr.getName());
	if (p != 0) p->sendCommand(KPrinter::Enable);

	return true;
}

//---------------------------------------------------------------------------
bool KPrinterManager::addReal(KPrinterDescription& descr)
{
	QString uri;
	uri = Kups_Infos::get()->getServerPortIppUri() + "/printers/" + descr.getName();

	// first block : Add printer without PPD
	{
		Kups_IppRequest request;
		request.setOperation(CUPS_ADD_PRINTER);
	  request.setUri("printer-uri", uri);

	  QString	duri = descr.getDeviceUri();
	  if (!duri.isEmpty()) request.setUri(IPP_TAG_PRINTER, "device-uri", duri);
#ifdef DEBUG
debug("device-uri : %s",descr.getDeviceUri().data());
#endif
		request.setInteger(IPP_TAG_PRINTER, "printer-state", IPP_PRINTER_IDLE);
		request.setBoolean(IPP_TAG_PRINTER, "printer-is-accepting-jobs", 1);
	 	if (!descr.getDescription().isNull()) request.setText(IPP_TAG_PRINTER, "printer-info", descr.getDescription() );
	 	if (!descr.getLocation().isNull()) request.setText(IPP_TAG_PRINTER, "printer-location", descr.getLocation() );
		if (!descr.getStartBanner().isNull() || !descr.getEndBanner().isNull())
		{
			QStringList	list;
			list.append((descr.getStartBanner().isEmpty() ? QString("none") : descr.getStartBanner()));
			list.append((descr.getEndBanner().isEmpty() ? QString("none") : descr.getEndBanner()));
			request.setNames(IPP_TAG_PRINTER, "job-sheets-default", list);
		}

	  if ( !request.doRequest("/admin/") )
	  {
			return false;
		}
	}
	// first block

  // second block : set the PPD to added printer
  {
	  Kups_IppRequest request;
	  request.setOperation(CUPS_ADD_PRINTER);
		request.setUri("printer-uri", uri);

	  if (descr.getBuffer() == 0)
	  {
			QString ppdFile = descr.getPPD();
			if (ppdFile.isEmpty())
	  	{
		  	setLastError("Raw printer added !");
		  	// do nothing more, this allows this function to be used for printer edition
		  	//return false;
	  	}
	  	else
	  	{
				if (ppdFile.at(0) != '/')
				{
	  			// Rel path
	  			ppdFile = Kups_Infos::get()->getDriversDir() + "/";
					ppdFile += descr.getPPD();
				}
				if ( !request.doFileRequest("/admin/", ppdFile) )
				{
					return false;
				}
			}
	  }
  	else // buffer == 0
  	{
			KBuffer* buffer = descr.getBuffer();
#ifdef DEBUG
//debug("%s", buffer->data() );
#endif
			if ( !request.doBufferRequest("/admin/", buffer) )
			{
				return false;
			}
	  }
  }
  // second block

	refresh(); // so we have the new added printer

	KPrinter* p = getPrinterByName( descr.getName() );
	if (p != 0)
	{
		p->sendCommand(KPrinter::Enable);
	}
		
	return true;
}

bool KPrinterManager::refresh()
{
	bool	value(true);

	setAllClassesDiscarded(true);
	setAllJobsDiscarded(true);
	setAllPrintersDiscarded(true);

	if (!refreshPrinters())
	{
  		value = false;
	}

	if (value && !refreshClasses())
	{
  		value = false;
	}
	
	if (value && !refreshDefault())
	{
		value = false;
	}
	
	if (value && !refreshJobs())
	{
		value = false;
	}

	if (value && !KInstanceManager::get()->refresh())
	{
		value = false;
	}
	
	notifyObservers();

	deleteAllDiscardedPrinters();
	deleteAllDiscardedClasses();
	deleteAllDiscardedJobs();

	return value;
}

bool KPrinterManager::refreshClasses()
{
	Kups_IppRequest request;
	request.setOperation(CUPS_GET_CLASSES);

	
  if (request.doRequest("/classes/"))
  {
		Kups_AttributesIterator* iter = request.getAttributesIterator();
		KClassPrinter *classe = new KClassPrinter();
	
		ipp_attribute_t *attr;
		while ( (attr = iter->next()) != 0 )
  	{
   		classe->setAttribute(attr);
   		if (attr->name == NULL || attr == iter->last())
   		{
   			if (!classe->getName().isEmpty())
   			{
   				KClassPrinter	*aClasse = getClassByUri(classe->getIppURI());
   				if (aClasse)
   				{
   					aClasse->copy(classe);
   					aClasse->setIsDiscarded(false);
   					delete classe;
   					classe = 0;
   				}
   				else
   				{
   					addClassToList(classe);
   				}
   				classe = new KClassPrinter();
   			}
   			else
   			{
   				delete classe;
   				classe = new KClassPrinter();
   			}
   		}
		}; // while attr != 0;
		delete classe;
  }
  else
  {
  	if (getLastError().isEmpty())
  	{
  		setLastError("Can't get response to IPP request");
  	}
  	return false;
  } // end if cupsdDoRequest

  return true;
}

bool KPrinterManager::refreshJobs()
{
	Kups_IppRequest request;
	request.setOperation(IPP_GET_JOBS);
	QString uri;
	uri = Kups_Infos::get()->getServerPortIppUri() + "/jobs/";
	request.setUri("job-uri", uri);

	
  if (request.doRequest("/jobs/"))
  {
		Kups_AttributesIterator* iter = request.getAttributesIterator();
		KJob *job = new KJob();
	
		ipp_attribute_t *attr;	/* Current attribute */
		while ( (attr = iter->next()) != 0 )
  	{
   		job->setAttribute(attr);
   		if ( attr->name == NULL || attr == iter->last())
  		{
  			if (job->getID() != -1 && job->getPrinter())
  			{
  				KJob* aJob = getJobByID( job->getID() );
  				if (aJob)
  				{
  					aJob->copy(job);
  					aJob->setIsDiscarded(false);
  					delete job;
  					job = 0;
					}
					else
					{
						job->getPrinter()->addJobToList(job);
					}
					job = new KJob();
				}
				else
				{
					delete job;
					job = new KJob();
				}
			}
		}; // while attr != 0;
		delete job;
  }
  else
  {
  	if (getLastError().isEmpty())
  	{
  		setLastError("Can't get response to IPP request");
  	}
  	return false;
  } // end if cupsdDoRequest


  return true;
}

bool KPrinterManager::refreshDefault()
{
	Kups_IppRequest request;
	request.setOperation(CUPS_GET_DEFAULT);

	ipp_attribute_t* attr = 0;
  if (request.doRequest("/printers/"))
  {
    attr = request.findAttribute("printer-uri", IPP_TAG_URI);
    if (!attr) attr = request.findAttribute("printer-uri-supported", IPP_TAG_URI);
    if (attr)
    {
    	KPrinterManager::get()->setDefaultPrinterUri(QString(attr->values[0].string.text));
    }
    else
    {
    	attr = request.findAttribute("printer-name", IPP_TAG_NAME);
    	if (attr)
    	{
    		KPrinterManager::get()->setDefaultPrinterName(QString(attr->values[0].string.text));
    	}
    	else
    	{
				setLastError(i18n("Can't get default destination"));
				return  false;
    	}
    }
    return true;
  }
  return false;
}

//-------------------------------------------------------------------------
bool KPrinterManager::refreshPrinters()
{
	Kups_IppRequest request;
	request.setOperation(CUPS_GET_PRINTERS);


  if (request.doRequest("/printers/"))
  {
		Kups_AttributesIterator* iter = request.getAttributesIterator();
		KRealPrinter *printer = new KRealPrinter();

		ipp_attribute_t *attr;
		while ( (attr = iter->next()) != 0 )
  	{
   		printer->setAttribute(attr);
   		if (attr->name == NULL || attr == iter->last())
   		{
   			if (!printer->getName().isEmpty())
   			{
   				KRealPrinter	*aPrinter = getPrinterByUri(printer->getIppURI());
   				if (aPrinter)
   				{
   					aPrinter->copy(printer);
   					aPrinter->setIsDiscarded(false);
   					delete printer;
   					printer = 0;
   				}
   				else
   				{
   					addPrinterToList(printer);
   				}
   				printer = new KRealPrinter();
   			}
   			else
   			{
   				delete printer;
   				printer = new KRealPrinter();
   			}
   		}
		}; // while attr != 0;
		delete printer;
  }
  else
  {
  	if (getLastError().isEmpty())
  	{
  		setLastError("Can't get response to IPP request");
  	}
  	return false;
  } // end if cupsdDoRequest

  return true;
}
