/***************************************************************************
                          kups_ipprequest.cpp  -  description
                             -------------------
    begin                : Sun Nov 28 1999
    copyright            : (C) 1999 by jec
    email                : cuendet@linkvest.ch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include <stdlib.h>
#include <ctype.h>
#include <errno.h>
#include <sys/stat.h>

#include <qstring.h>
#include <qfile.h>
#include <qtextstream.h>
#include <klocale.h>

#include <cups/language.h>
#include <cups/cups.h>
#include <cups/ipp.h>

#include "kbuffer.h"

#include "kups_ipprequest.h"
#include "kprintermanager.h"
#include "kups_infos.h"
#include "kups_attributesiterator.h"

void dumpRequest(ipp_t *req)
{
	debug("------------------------------");
	debug("state = 0x%x",req->state);
	debug("current tag = 0x%x",req->curtag);
	debug("request operation ID = 0x%x",req->request.op.operation_id);
	debug("request ID = 0x%x",req->request.op.request_id);
	debug("request version = %c%c",req->request.op.version[0],req->request.op.version[1]);
	ipp_attribute_t	*attr = req->attrs;
	while (attr)
	{
		debug(" ");
		debug("attribute: %s",attr->name);
		debug("group tag = 0x%x",attr->group_tag);
		debug("value tag = 0x%x",attr->value_tag);
		debug("number of values = %d",attr->num_values);
		if (attr->value_tag >= IPP_TAG_TEXT)
			for (int i=0;i<attr->num_values;i++)
				debug("value[%d] = %s",i,attr->values[i].string.text);
		attr = attr->next;
	}
	debug("------------------------------");
}

Kups_IppRequest::Kups_IppRequest()
:http_(0),
request_(0),
response_(0),
attributesIterator_(0)
{
	request_ = ippNew();
  request_->request.op.request_id   = 1;
  request_->request.op.operation_id = (ipp_op_t)-1;

  cups_lang_t	*language = cupsLangDefault();

  ippAddString(request_, IPP_TAG_OPERATION, IPP_TAG_CHARSET,
               "attributes-charset", NULL, cupsLangEncoding(language));

  ippAddString(request_, IPP_TAG_OPERATION, IPP_TAG_LANGUAGE,
               "attributes-natural-language", NULL, language->language);


}
Kups_IppRequest::~Kups_IppRequest()
{
  httpClose(http_);

	if (request_) ippDelete(request_);
	if (response_) ippDelete(response_);
	
	delete attributesIterator_;
}

void Kups_IppRequest::setOperation(ipp_op_t op)
{
  request_->request.op.operation_id = op;
}



void Kups_IppRequest::setUri(const QString& uriType, const QString& uri)
{
	setUri(IPP_TAG_OPERATION, uriType, uri);
}
void Kups_IppRequest::setUris(const QString& uriType, const QStringList& uris)
{
	setUris(IPP_TAG_OPERATION, uriType, uris);
}
/*void Kups_IppRequest::setUriAt(const QString& uriType, int index, const QString& uri)
{
	setUriAt(IPP_TAG_OPERATION, uriType, index, uri);
}*/
void Kups_IppRequest::setInteger(const QString& uriType, int value)
{
	setInteger(IPP_TAG_OPERATION, uriType, value);
}
void Kups_IppRequest::setBoolean(const QString& uriType, bool value)
{
	setBoolean(IPP_TAG_OPERATION, uriType, value);
}
void Kups_IppRequest::setMime(const QString& uriType, const QString& mime)
{
	setMime(IPP_TAG_OPERATION, uriType, mime);
}
void Kups_IppRequest::setName(const QString& uriType, const QString& name)
{
	setName(IPP_TAG_OPERATION, uriType, name);
}
void Kups_IppRequest::setNames(const QString& uriType, const QStringList& uris)
{
	setUris(IPP_TAG_OPERATION, uriType, uris);
}
void Kups_IppRequest::setText(const QString& uriType, const QString& name)
{
	setName(IPP_TAG_OPERATION, uriType, name);
}
	
void Kups_IppRequest::setUri(ipp_tag_t type, const QString& uriType, const QString& uri)
{
  ippAddString(request_, type, IPP_TAG_URI, uriType.data(), NULL, uri.data());
}
/*void Kups_IppRequest::setUriAt(ipp_tag_t type, const QString& uriType, int index, const QString& uri)
{
	ipp_attribute_t* attr = ippAddStrings(request_, type, IPP_TAG_URI, uriType.data(), index+1, NULL, NULL);

	attr->values[index].string.text = strdup( uri.data() );
}*/
void Kups_IppRequest::setUris(ipp_tag_t type, const QString& uriType, const QStringList& uris)
{
	ipp_attribute_t	*attr = ippAddStrings(request_, type, IPP_TAG_URI, uriType.data(), uris.count(), NULL, NULL);
	for (int i=0;i<(int)(uris.count());i++)
		attr->values[i].string.text = strdup( (*uris.at(i)).data() );
}
void Kups_IppRequest::setInteger(ipp_tag_t type, const QString& uriType, int value)
{
  ippAddInteger(request_, type, IPP_TAG_INTEGER, uriType.data(), value);
}
void Kups_IppRequest::setBoolean(ipp_tag_t type, const QString& uriType, bool value)
{
	ippAddBoolean(request_, type, uriType.data(), value);
}
void Kups_IppRequest::setMime(ipp_tag_t type, const QString& uriType, const QString& mime)
{
  ippAddString(request_, type, IPP_TAG_MIMETYPE, uriType.data(), NULL, (mime.isEmpty() ? "" : mime.data()));
}
void Kups_IppRequest::setName(ipp_tag_t type, const QString& uriType, const QString& name)
{
  ippAddString(request_, type, IPP_TAG_NAME, uriType.data(), NULL, (name.isEmpty() ? "" : name.data()));
}
void Kups_IppRequest::setNames(ipp_tag_t type, const QString& uriType, const QStringList& uris)
{
	ipp_attribute_t	*attr = ippAddStrings(request_, type, IPP_TAG_NAME, uriType.data(), uris.count(), NULL, NULL);
	for (int i=0;i<(int)(uris.count());i++)
		attr->values[i].string.text = strdup( (*uris.at(i)).data() );
}
void Kups_IppRequest::setText(ipp_tag_t type, const QString& uriType, const QString& text)
{
  ippAddString(request_, type, IPP_TAG_TEXT, uriType.data(), NULL, (text.isEmpty() ? "" : text.data()));
}



Kups_AttributesIterator* Kups_IppRequest::getAttributesIterator()
{
	if (attributesIterator_ == 0)
	{
		attributesIterator_ = new Kups_AttributesIterator;
	}
	attributesIterator_->setResponse(response_);
	return attributesIterator_;
}

ipp_attribute_t* Kups_IppRequest::findAttribute(const QString& attrName, ipp_tag_t)
{
	Kups_AttributesIterator* iter = getAttributesIterator();
	ipp_attribute_t* attr = 0;
	while( (attr = iter->next()) != 0 )
	{
		if (attrName == attr->name)
		{
			break;
		}
	}
	return attr;
}

bool Kups_IppRequest::doRequest(const QString& resource)
{
	return doBufferRequest(resource, 0);
}

bool Kups_IppRequest::doStringRequest(const QString& res, const QString& str)
{
	KBuffer buffer;
	buffer.assign(str);
	return doBufferRequest(res, &buffer);
}

bool Kups_IppRequest::doBufferRequest(const QString& res, const KBuffer* buffer)
{
	bool	retVal(false);
	if (buffer)
	{
		QString	tmpFile = cupsTempFile(NULL,0);
		if (!buffer->dump(tmpFile))
		{
			QString	msg = i18n("Can't open file '%1'").arg(tmpFile);
			KPrinterManager::setLastError(msg);
			unlink(tmpFile.data());
			return false;
		}
		retVal = doFileRequest(res,tmpFile);
		unlink(tmpFile.data());
	}
	else retVal = doFileRequest(res,QString(""));
	return retVal;
}

bool Kups_IppRequest::doFileRequest(const QString& res, const QString& fileName)
{
	if (request_->request.op.operation_id == -1)
	{
		KPrinterManager::setLastError("No IPP operation specified");
		return false;
	}

	httpClose(http_);

	http_ = httpConnect( Kups_Infos::get()->getServer().data(), Kups_Infos::get()->getPort() );
	if (http_ == 0)
	{
	  	QString str = i18n("Can't connect to server <%1> on port <%2>\nPerhaps CUPSD is not started ?").arg(Kups_Infos::get()->getServer()).arg(Kups_Infos::get()->getPort());
	  	KPrinterManager::setLastError(str);
	  	return false;
	}

	if (response_)
	{
		ippDelete(response_);
		response_ = 0;
	}

#ifdef DEBUG
#define	IPP_REQ_MASK	CUPS_ADD_PRINTER
//if (request_->request.op.operation_id == IPP_REQ_MASK) dumpRequest(request_);
#endif

  	response_ = cupsDoFileRequest(http_,request_,res.data(),(fileName.isEmpty() ? NULL : fileName.data()));
  	request_ = 0;	// request_ has been deleted during request processing

	if (!response_ || response_->state == IPP_ERROR)
	{
		QString	msg = (response_ ? QString(ippErrorString(getResponseStatus())) : i18n("Unable to send request"));
		KPrinterManager::setLastError(msg);
		return false;
	}
#ifdef DEBUG
#define	DUMP_ANSWER	0
if (DUMP_ANSWER) dumpRequest(response_);
#endif
  	return true;
}

ipp_status_t Kups_IppRequest::getResponseStatus()
{
	return (response_ ? response_->request.status.status_code : IPP_OK);
}
